% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/AllGenerics.R
\name{storeResults}
\alias{storeResults}
\title{Save xcms result objects in a specified format}
\usage{
storeResults(object, param, ...)
}
\arguments{
\item{object}{\code{MsExperiment} or \code{XcmsExperiment} The data object that needs
to be saved.}

\item{param}{The parameter object selecting and configuring the format for
saving. It can be one of the following classes: \code{\link{RDataParam}},
\code{\link{PlainTextParam}}, or \code{MzTabMParam}.}

\item{...}{Optional parameters.}
}
\description{
The \code{storeResults} function saves an \code{object} resulting from processing with
the \code{xcms} package (mainly \code{XcmsExperiment}). Multiple formats for storing
and exporting are available and can be defined by the \code{param} argument.

Supported \code{param} objects are:
\itemize{
\item \code{\link{RDataParam}}: Save in an .RData format file. The name of the file can be
specified in the \code{fileName} argument.
\item \code{\link{PlainTextParam}}: Store \code{MsExperiment} and \code{XcmsExperiment} objects as a
folder of plain text files, folder path defined in the \code{path} argument.
\item \code{MzTabMParam}: Save in MzTab format (to be defined).
}

For specific examples, see the help pages of the individual parameter classes
listed above.
}
\examples{

## Load a test data set with detected peaks
library(xcms)
library(MsExperiment)
faahko_sub <- loadXcmsData("faahko_sub2")

## Set up parameter to save as .RData file
param <- RDataParam(fileName = "example_xcms_results")

## save as .RData
storeResults(object = faahko_sub, param = param)

## Set up parameter to save as a collection of plain text file
param <- PlainTextParam(path = "test/path/")

## Save as a collection of plain text files
storeResults(object = faahko_sub, param = param)

}
\author{
Philippine Louail
}
